package scripts.migration.from4_4To4_5.general;

import javax.jcr.ImportUUIDBehavior;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.context.Context;

import info.magnolia.importexport.BootstrapUtil;

import info.magnolia.cms.util.ClasspathResourcesUtil;

import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

/**
 * The intention of this script is to add logic, which has to be executed after the complete migration has been done.
 * For example adding new dialog controls, new properties to template definitions and so on.
 */
class PostMigrationProcessing{

    Context ctx
    HierarchyManager hm

    /**
     * Executes all property changes defined in this script.
     * Any additional property changes can be added to this method for execution.
     */
    def executePropertyChanges(){
        changeCreateOrDeleteProperties("config", mapOfPropertiesToChangeInConfig())
        changeCreateOrDeleteProperties("config", mapOfAreaTitlesAndDescriptionsToAdd())
    }

    /**
     * Executes all nodes to remove.
     * Any additional nodes to be removed can be added to this method for execution.
     */
    def executeDeleteNodes(){
        deleteNodes("config", listOfEmptyNodesToDelete(), true)
        deleteNodes("config", listOfNotEmptyNodesToDelete(), false)
    }

    /**
    * Executes all properties to remove.
    * Any additional properties to be removed can be added to this method for execution.
    */
   def executeDeleteProperties(){
       deleteProperties("website", listOfWebsitePropertiesToDelete())
       deleteProperties("config", listOfConfigPropertiesToDelete())
   }

    /**
     * Executes the import of all bootstraps defined in this script.
     * Any additional bootstraps to import can be added to this method for execution.
     */
    def executeImportBootstraps(){
        bootstrapResources("config", findResources("/auto_bootstraps_after_migration/config/.*"), ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW)
//        bootstrapResources("website", findResources("/auto_bootstraps_after_migration/website/.*"), ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW)
        bootstrapResources("config", manualBoostrapsAfterImport(), ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW)
    }

    /**
     * Returns all resources within the ResourcesClassPath found by the pattern.
     * @param pattern Pattern to filter by.
     * @return All found resources.
     */
    def static String[] findResources(String pattern){
        return ClasspathResourcesUtil.findResources(pattern)
    }

    /**
     * This method will change or create any property passed within the map.
     * If the path to the property does not exist, the parent path will be created as CONTENTNODE.
     * If the property value in the map is EMPTY, then the property will be deleted.
     * If no property remains in the parent node, the parent node will also be deleted.
     *
     * @param workspace The workspace to operate in.
     * @param propertiesToChange A map of the properties to change. Key is the path, value is the property value.
     */
    def changeCreateOrDeleteProperties(String workspace, Map propertiesToChange){
        hm = ctx.getHierarchyManager(workspace)

        propertiesToChange.each{ String propertyPath, String propertyValue ->

            String parentPathOfProperty = StringUtils.substringBeforeLast(propertyPath, "/")
            String propertyName = StringUtils.substringAfterLast(propertyPath, "/")

            if(StringUtils.isNotEmpty(propertyValue)){
                Content parenNodeOfProp = ContentUtil.getOrCreateContent(hm.getRoot(), StringUtils.substringAfter(parentPathOfProperty, "/"), ItemType.CONTENTNODE)
                parenNodeOfProp.setNodeData(propertyName, propertyValue)

            } else { //Remove property if value is empty String
                if(hm.isExist(parentPathOfProperty)){
                    Content parenNodeOfProp = hm.getContent(parentPathOfProperty)
                    if(parenNodeOfProp.hasNodeData(propertyName)){
                        parenNodeOfProp.deleteNodeData(propertyName)
                    }
                    //Remove node if no properties exist anymore
                    if(parenNodeOfProp.getNodeDataCollection().size() == 0){
                        parenNodeOfProp.delete();
                    }
                }
            }

        }
    }

    /**
     * This method deletes all nodes passed within the collection.
     * It can be decided by the @param deleteOnlyEmptyNodes if only empty nodes should be deleted.
     *
     * @param workspace Operating in workspace.
     * @param nodesToDelete All paths of the nodes to delete.
     * @param deleteOnlyEmptyNodes If true, only empty nodes will be deleted.
     */
    def deleteNodes(String workspace, Collection nodesToDelete, boolean deleteOnlyEmptyNodes){
        hm = ctx.getHierarchyManager(workspace)
        nodesToDelete.each{ String nodePath ->
            if(hm.isExist(nodePath)){
                Content nodeToDetele = hm.getContent(nodePath)
                boolean isEmpty = !nodeToDetele.hasChildren() && nodeToDetele.getNodeDataCollection().size()==0
                if(isEmpty || !deleteOnlyEmptyNodes){
                    nodeToDetele.delete();
                }
            }
        }
    }

    /**
     * This method deletes all properties passed within the collection.
     *
     * @param workspace Operating in workspace.
     * @param propertiesToDelete All paths of the properties to delete.
     */
    def deleteProperties(String workspace, Collection propertiesToDelete){
        hm = ctx.getHierarchyManager(workspace)
        propertiesToDelete.each{
            if(hm.isExist(it)){
                hm.delete(it)
            }
        }
    }

    /**
     * Will import all bootstrap files defined within the passed Sting array.
     *
     * @param workspace The workspace to operate on.
     * @param pathsToBoostraps A String array of paths of the bootstraps to import.
     * @param importUUIDBehavior The uuid behavior on import.
     */
    def bootstrapResources(String workspace, String[] pathsToBoostraps, int importUUIDBehavior){
        hm = ctx.getHierarchyManager(workspace)
        BootstrapUtil.bootstrap(pathsToBoostraps, importUUIDBehavior);
    }

    /**
     * A general map of all properties to change within the config workspace.
     */
    def static Map mapOfPropertiesToChangeInConfig(){
        Map propertiesToChange = [
            //STK
            "/modules/standard-templating-kit/templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabbed/areas/horizontalTabItems/parameters/olClass" : "text-box-toc",
            "/modules/standard-templating-kit/templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabbed/areas/horizontalTabItems/parameters/divClass" : "text-box-section-group",
            "/modules/standard-templating-kit/templates/components/extras/horizontalTabbed/stkExtrasHorizontalTabbed/areas/horizontalTabItems/parameters/olClass" : "toc-box-toc",
            "/modules/standard-templating-kit/templates/components/extras/horizontalTabbed/stkExtrasHorizontalTabbed/areas/horizontalTabItems/parameters/divClass" : "toc-box-section-group",
            "/modules/standard-templating-kit/templates/components/content/stkTextImage/parameters/divClass" : "text-section",
            "/modules/standard-templating-kit/templates/components/features/stkImageGallery/parameters/dlClass" : "media photo",
            "/modules/standard-templating-kit/dialogs/components/content/stkDownloadList/tabQuery/label" : "Download List",
            "/modules/standard-templating-kit/config/themes/pop/imaging/class" : "info.magnolia.module.templatingkit.imaging.STKImagingSupport",


            "/modules/standard-templating-kit/templates/components/teasers/stkTeaser/areas/linkList/optional" : "true",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaser/areas/linkList/dialog" : "standard-templating-kit:components/teasers/stkLinkListArea",

            //Form
            "/modules/form/templates/components/formGroupFields/title" : "paragraph.formGroupFields.title",
            "/modules/form/templates/components/form/title" : "paragraph.form.title",
            "/modules/form/templates/components/form/description" : "paragraph.form.description",
            //Shop
            "/modules/shop/templates/components/features/form/shopConfirmTerms/extends" : "/modules/form/templates/components/formEdit"
            ]
    }

    /**
     * A map of all area titles and descriptions to add to STK's areas.
     */
    def static Map mapOfAreaTitlesAndDescriptionsToAdd() {
        Map titlesAndDescriptionsToAdd = [
            //prototype
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/htmlHeader/title" : "areas.templates.htmlHeader.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/htmlHeader/description" : "areas.templates.htmlHeader.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/platform/title" : "areas.templates.platform.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/platform/description" : "areas.templates.platform.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/title" : "areas.templates.main.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/description" : "areas.templates.main.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/intro/title" : "areas.templates.main.intro.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/intro/description" : "areas.templates.main.intro.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/intro/areas/infoBlock/title" : "areas.templates.main.intro.infoBlock.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/intro/areas/infoBlock/description" : "areas.templates.main.intro.infoBlock.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/comments/title" : "areas.templates.main.comments.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/comments/description" : "areas.templates.main.comments.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/content/title" : "areas.templates.main.content.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/content/description" : "areas.templates.main.content.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/breadcrumb/title" : "areas.templates.main.breadcrumb.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/breadcrumb/description" : "areas.templates.main.breadcrumb.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/contentNavigation/title" : "areas.templates.main.contentNavigation.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/main/areas/contentNavigation/description" : "areas.templates.main.contentNavigation.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/title" : "areas.templates.extras.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/description" : "areas.templates.extras.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/areas/module/title" : "areas.templates.extras.module.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/areas/module/description" : "areas.templates.extras.module.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/areas/extras1/title" : "areas.templates.extras.extras1.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/areas/extras1/description" : "areas.templates.extras.extras1.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/areas/extras2/title" : "areas.templates.extras.extras2.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/extras/areas/extras2/description" : "areas.templates.extras.extras2.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/promos/title" : "areas.templates.promos.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/promos/description" : "areas.templates.promos.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/base/title" : "areas.templates.base.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/base/description" : "areas.templates.base.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/footer/title" : "areas.templates.footer.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/footer/description" : "areas.templates.footer.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/sectionHeader/title" : "areas.templates.sectionHeader.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/sectionHeader/description" : "areas.templates.sectionHeader.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/stage/title" : "areas.templates.stage.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/stage/description" : "areas.templates.stage.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/title"                            : "areas.templates.branding.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/description"                      : "areas.templates.branding.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/areas/metaNavigation/title"       : "areas.templates.branding.metaNavigation.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/areas/metaNavigation/description" : "areas.templates.branding.metaNavigation.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/areas/logo/title"                 : "areas.templates.branding.logo.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/areas/logo/description"           : "areas.templates.branding.logo.description",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/areas/search/title"               : "areas.templates.branding.search.title",
            "/modules/standard-templating-kit/config/site/templates/prototype/areas/branding/areas/search/description"         : "areas.templates.branding.search.description",
            //pages/demo-features
            //pages
            //components
            //pur
            "/modules/standard-templating-kit/templates/components/pur/stkPURUpdateForm/areas/fieldsets/title" : "areas.components.pur.fieldsets.title",
            "/modules/standard-templating-kit/templates/components/pur/stkPURUpdateForm/areas/fieldsets/description" : "areas.components.pur.fieldsets.description",
            "/modules/standard-templating-kit/templates/components/pur/stkPURPasswordForm/areas/fieldsets/title" : "areas.components.pur.fieldsets.title",
            "/modules/standard-templating-kit/templates/components/pur/stkPURPasswordForm/areas/fieldsets/description" : "areas.components.pur.fieldsets.description",
            "/modules/standard-templating-kit/templates/components/pur/stkPURRegistrationForm/areas/fieldsets/title" : "areas.components.pur.fieldsets.title",
            "/modules/standard-templating-kit/templates/components/pur/stkPURRegistrationForm/areas/fieldsets/description" : "areas.components.pur.fieldsets.description",
            "/modules/standard-templating-kit/templates/components/pur/groupFields/areas/fields/title" : "areas.components.pur.fields.title",
            "/modules/standard-templating-kit/templates/components/pur/groupFields/areas/fields/description" : "areas.components.pur.fields.description",
            //stk
            "/modules/standard-templating-kit/templates/components/content/stkLinkList/areas/linkList/title" : "areas.components.linkList.title",
            "/modules/standard-templating-kit/templates/components/content/stkLinkList/areas/linkList/description" : "areas.components.linkList.description",
            "/modules/standard-templating-kit/templates/components/features/stkFAQ/areas/faqItems/title" : "areas.components.faq.faqItems.title",
            "/modules/standard-templating-kit/templates/components/features/stkFAQ/areas/faqItems/description" : "areas.components.faq.faqItems.description",
            "/modules/standard-templating-kit/templates/components/footer/stkFooterLinkList/areas/linkList/title" : "areas.components.linkList.title",
            "/modules/standard-templating-kit/templates/components/footer/stkFooterLinkList/areas/linkList/description" : "areas.components.linkList.description",
            "/modules/standard-templating-kit/templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabbed/areas/horizontalTabItems/title" : "areas.components.horizontalTabItems.title",
            "/modules/standard-templating-kit/templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabbed/areas/horizontalTabItems/description": "areas.components.horizontalTabItems.description",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaser/areas/linkList/title" : "areas.components.linkList.title",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaser/areas/linkList/description" : "areas.components.linkList.description",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserGroup/areas/teaserGroupItems/title" : "areas.components.teaserGroupItems.title",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserGroup/areas/teaserGroupItems/description" : "areas.components.teaserGroupItems.description",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserCarousel/areas/carouselItems/title" : "areas.components.carouselItems.title",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserCarousel/areas/carouselItems/description" : "areas.components.carouselItems.description",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserPureLinkList/areas/linkList/title" : "areas.components.linkList.title",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserPureLinkList/areas/linkList/description" : "areas.components.linkList.description",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserFingerTabbed/areas/fingerTabItems/title" : "areas.components.fingerTabItems.title",
            "/modules/standard-templating-kit/templates/components/teasers/stkTeaserFingerTabbed/areas/fingerTabItems/description" : "areas.components.fingerTabItems.description",
            //other modules
            //form
            "/modules/form/templates/components/form/areas/fieldsets/title" : "areas.components.form.fieldsets.title",
            "/modules/form/templates/components/form/areas/fieldsets/description" : "areas.components.form.fieldsets.description",
            "/modules/form/templates/components/formSubmit/areas/conditionList/title" : "areas.components.form.formSubmit.conditionList.title",
            "/modules/form/templates/components/formSubmit/areas/conditionList/description" : "areas.components.form.formSubmit.conditionList.description",
            "/modules/form/templates/components/formGroupEdit/areas/edits/title" : "areas.components.form.formGroupEdit.edits.title",
            "/modules/form/templates/components/formGroupEdit/areas/edits/description" : "areas.components.form.formGroupEdit.edits.description",
            "/modules/form/templates/components/formGroupFields/areas/fields/title" : "areas.components.form.formGroupFields.fields.title",
            "/modules/form/templates/components/formGroupFields/areas/fields/description" : "areas.components.form.formGroupFields.fields.description"
            ];
    }

   /**
    * A list of all nodes that have to be deleted even if not empty.
    */
   def static Collection listOfNotEmptyNodesToDelete(){
       Collection listOfEmptyNodesToDelete = [
           "/modules/standard-templating-kit/templates/pages/stkArticle/areas/main/areas/content/availableComponents/stkTeaserFingerTabbed",
           "/modules/standard-templating-kit/templates/pages/stkLargeArticle/areas/main/areas/content",
           "/modules/standard-templating-kit/templates/pages/stkNews/areas/main/areas/content",
           "/modules/standard-templating-kit/templates/pages/stkEvent/areas/main/areas/content",
           "/modules/standard-templating-kit/templates/pages/stkGlossaryTerm/areas/main/areas/content",
           "/modules/standard-templating-kit/templates/components/teasers/horizontalTabbed/stkTeaserHorizontalTabitemInternalPage/linkList",
           "/modules/form/dialogs/form/tabMain/displayBreadcrumb",
           "/modules/standard-templating-kit/dialogs/components/teasers/stkTeaser/tabLinkList",
           "/modules/standard-templating-kit/dialogs/components/teasers/stkTeaserFingerTabbed/tabTeaser/dummy"

           ];
       return listOfEmptyNodesToDelete
   }

    /**
     * A list of all nodes that have to be deleted if empty.
     */
    def static Collection listOfEmptyNodesToDelete(){
        Collection listOfEmptyNodesToDelete = [
            "/modules/standard-templating-kit/templates/pages/demo-features/stkSectionWithTwoExtras/areas/extras/areas",
            "/modules/standard-templating-kit/templates/components/footer/stkFooterLinkList/parameters"
            ];
        return listOfEmptyNodesToDelete
    }

    def static Collection listOfWebsitePropertiesToDelete(){
        Collection properties = [
            "/demo-project/multimedia/video-player/main/content/0/height",
            "/demo-project/multimedia/video-player/main/content/0/width",
            "/demo-features/content-templates/article/main/content/03/height",
            "/demo-features/content-templates/article/main/content/03/width"
        ]
        return properties
    }

    def static Collection listOfConfigPropertiesToDelete(){
        Collection properties = [
            "/modules/standard-templating-kit/templates/pages/stkImageGallery/areas/main/showIntroEditBar"
        ]
        return properties
    }

    /**
     * A String array of all bootstrap files to import manually defined in this map.
     */
    def static String[] manualBoostrapsAfterImport(){
        String[] bootstrapsToImport = [
            //"/manual_bootstraps_after_migration/path_to_the_xml_bootsrapfile",
            ]
        return bootstrapsToImport
    }

}
