package scripts.migration.from4_4To4_5.stk;

import scripts.migration.from4_4To4_5.util.MigrationUtil;
import groovy.text.Template;
import info.magnolia.cms.core.Content
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.context.Context;
/**
 * Since STK 2.0, the configuration of the template is more "flat". There are less inner areas.
 * Areas flattened:
 * 	- header: stage, sectionHeader and branding move to the top. header is removed
 *
 */
class PreProcessSTKTemplates {
  Context ctx
  HierarchyManager hm
  String workspace

  /**
   * Flatten a stk template definition:
   * 1. The header sub area definition are moved up to the top level, because they are now top level areas.
   * 2. Move the meta navigation area to top level for being processes as an area.
   *
   * @param templateDef The template definition to flatten.
   */
  def flattenTemplate(Content templateDef){
    //Flattern the header area
    moveChildrenUp("header", templateDef)
    //Flattern the meta navigation
    if(templateDef.hasContent("navigation") && templateDef.getContent("navigation").hasContent("meta")){
      def upperLevelPath = templateDef.getHandle()
      def area = templateDef.getContent("navigation").getContent("meta")
      area = MigrationUtil.renameAndMergeNodes(area, area.getName()+"Navigation", false, true)
      area = MigrationUtil.moveAndMergeNodes(area, upperLevelPath+"/branding", false, true)
    }
  }

  /**
   * Move the children of the area in the specified level.
   *
   * @param areaName The name of the area.
   * @param template The node where the children have to be moved.
   */
  def private moveChildrenUp(String areaName, Content template){
    def upperLevelPath = template.getHandle()
    if(template.hasContent(areaName)){
      def area = template.getContent(areaName)
      ContentUtil.getAllChildren(area).each{ Content child ->
      MigrationUtil.moveAndMergeNodes(child, upperLevelPath, false, true)
      }
      area.delete()
    }
  }

  /**
   * Main area will not contain any paragraphs anymore. The main area will contain a sub area named 'content', which will contain all former main area content.
   * There for all paragraph relevant definitions are moved into the new created 'content' area.
   *
   * The breadcrumb will become a part of the mainArea's areas, so moving it up into the mainArea node.
   *
   * @param templateDef The template definition to transform.
   */
  def mainAreaToNewSubContentArea(Content templateDef){
      if(templateDef.hasContent("mainArea")){
          Content mainArea = templateDef.getContent("mainArea")
          Content contentArea = MigrationUtil.getOrCreateContent(mainArea, "content", ItemType.CONTENTNODE)
          contentArea.setNodeData("enabled", true)
          if(mainArea.hasContent("paragraphs")){
              MigrationUtil.moveAndMergeNodes(mainArea.getContent("paragraphs"), mainArea.getHandle()+"/"+contentArea.getName(), false, true)
//              ContentUtil.moveInSession(mainArea.getContent("paragraphs"), mainArea.getHandle()+"/"+contentArea.getName()+"/paragraphs")
          }
          if(mainArea.hasContent("autoGeneratedParagraph")){
              MigrationUtil.moveAndMergeNodes(mainArea.getContent("autoGeneratedParagraph"), mainArea.getHandle()+"/"+contentArea.getName(), false, true)
//              ContentUtil.moveInSession(mainArea.getContent("autoGeneratedParagraph"), mainArea.getHandle()+"/"+contentArea.getName()+"/autoGeneratedParagraph")
          }
      }
      if(templateDef.hasContent("breadcrumb")){
          Content mainArea = MigrationUtil.getOrCreateContent(templateDef, "mainArea", ItemType.CONTENTNODE)
          MigrationUtil.moveAndMergeNodes(templateDef.getContent("breadcrumb"), mainArea.getHandle(), false, true)
//          ContentUtil.moveInSession(templateDef.getContent("breadcrumb"), mainArea.getHandle()+"/breadcrumb")
      }
  }

  /**
   * Changing the mainArea scripts for the prototype and content templates based on the map.
   * @param templateDef the template definition to process.
   */
  def changeMainAreaTemplates(Content templateDef){
      Map newMainAreaScripts = MigrationUtil.mapOfChangedMainAreaScriptsByTemplate()
      if(newMainAreaScripts.containsKey(templateDef.getName()) && templateDef.hasContent("mainArea")){
          templateDef.getContent("mainArea").setNodeData("template", newMainAreaScripts.get(templateDef.getName()))
      }
  }

  /**
   * Pre process the templates found in the module's template folder.
   */
  def preProcessModule(String moduleName){
    String moduleTemplatesPath = '/modules/'+moduleName+'/templates'
    if(hm.isExist(moduleTemplatesPath)){
      Content templates = hm.getContent(moduleTemplatesPath)
      Collection templateList = []
      MigrationUtil.visitTemplates(templates, templateList)
      templateList.each { Content templateDef ->
          flattenTemplate(templateDef)
          mainAreaToNewSubContentArea(templateDef)
          changeMainAreaTemplates(templateDef)
      }
    }
  }

  /**
   * Pre process the templates found in the siteDefinition's prototype/template folder.
   */
  def preProcessSiteDefintion(String siteDefintionPath) {
    String prototypePath = siteDefintionPath +"/templates/prototype"
    //If non EE the default site definition won't exist
    if(hm.isExist(prototypePath)){
        Content prototype = hm.getContent(prototypePath)
        flattenTemplate(prototype)
        mainAreaToNewSubContentArea(prototype)
        changeMainAreaTemplates(prototype)
    }
  }

  /**
   * Pre process all templates of all modules in the Collection.
   * @param moduleNames All module names to pre process their templates.
   */
  def preProcessModules(Collection moduleNames){
      moduleNames.each{ String moduleName ->
          preProcessModule(moduleName)
      }
  }

  /**
   * Pre process the site defintition's template/prototype.
   * @param siteDefinitionPaths All site definitions to Pre process its template/prototype.
   */
  def preProcessSiteDefinitions(Collection siteDefinitionPaths){
      siteDefinitionPaths.each{ String siteDefinitionPath ->
          preProcessSiteDefintion(siteDefinitionPath)
      }
  }

  /**
   * Pre process the module in the moduleNames list.
   *
   * Follow this example for your custom modules extending STK and all existing siteDefinitions.
   */
  def doPreProcess(Collection moduleNames, Collection siteDefinitionPaths) {
    hm = ctx.getHierarchyManager(workspace)
    preProcessModules(moduleNames)
    preProcessSiteDefinitions(siteDefinitionPaths)
  }

  /**
  * Pre process the module in the moduleNames list.
  *
  * Follow this example for your custom modules extending STK and all existing siteDefinitions.
  */
 def doPreProcessOfSTK() {
   hm = ctx.getHierarchyManager(workspace)
   preProcessModule("standard-templating-kit")
   preProcessSiteDefinitions(["/modules/standard-templating-kit/config/site", "/modules/extended-templating-kit/config/sites/default" ])
 }
}
